@@ -1,8 +1,6 @@
 import codecs
 import calendar
 import datetime
-import hashlib
-import hmac
 import io
 import json
 import logging
@@ -83,7 +81,13 @@ class AdsAPIError(Exception):
             self.str = self.message
             self.error = {'message': self.message}
         else:
-            self.message = self.error.get('error', {}).get('message')
+            error_dict = self.error.get('error', {})
+            self.message = error_dict.get('message', '')
+            # New error details in api 2.2
+            if 'error_user_title' in error_dict:
+                self.message += ' - ' + error_dict['error_user_title']
+            if 'error_user_msg' in error_dict:
+                self.message += ' - ' + error_dict['error_user_msg']
             self.code = self.error.get('error', {}).get('code')
             self.type = self.error.get('error', {}).get('type')
             self.str = '(%s %s) %s' % (self.type, self.code, self.message)
@@ -94,15 +98,22 @@ class AdsAPIError(Exception):
 
 class AdsAPI(object):
     """A client for the Facebook Ads API."""
-    DATA_LIMIT = 100
+    DATA_LIMIT = 1000
 
-    def __init__(self, access_token, app_id='', app_secret=''):
-        # Most ad account operations can be done without an app id and secret, so allow creating without them.
+    def __init__(self, access_token, app_id='', app_secret='', version=None):
+        """
+        :param access_token: The API access token
+        :param app_id: An optional App id, current only used for debug_token.
+        :param app_secret: An optional App secret, currently only used for debug_token
+        :param version: Facebook API version, e.g. "2.2". It's currently optional but will be required soon.
+        """
         self.access_token = access_token
         self.app_id = app_id
         self.app_secret = app_secret
-        h = hmac.new(access_token, app_secret, hashlib.sha256)
-        self.appsecret_proof = h.hexdigest()
+        if version:
+            self.api_root = '{}/v{}'.format(FACEBOOK_API, version)
+        else:
+            self.api_root = FACEBOOK_API
 
     def make_request(self, path, method, args=None, files=None, batch=False, raw_path=False):
         """Makes a request against the Facebook Ads API endpoint."""
@@ -116,15 +127,15 @@ class AdsAPI(object):
                 'method': method,
                 'relative_url': '%s?%s' % (path, urllib.urlencode(args))
             }
-        logger.info('Making a %s request at %s with %s' % (method, path, args))
+        logger.info('Making a %s request at %s/%s with %s' % (method, self.api_root, path, args))
         if 'access_token' not in args:
             args['access_token'] = self.access_token
         try:
             if method == 'GET':
-                url = path if raw_path else '%s/%s?%s' % (FACEBOOK_API, path, urllib.urlencode(args))
+                url = path if raw_path else '%s/%s?%s' % (self.api_root, path, urllib.urlencode(args))
                 f = urllib2.urlopen(url)
             elif method == 'POST':
-                url = path if raw_path else '%s/%s' % (FACEBOOK_API, path)
+                url = path if raw_path else '%s/%s' % (self.api_root, path)
                 if files:
                     encoder = MultipartFormdataEncoder()
                     content_type, body = encoder.encode(args, files)
@@ -134,7 +145,7 @@ class AdsAPI(object):
                 else:
                     f = urllib2.urlopen(url, urllib.urlencode(args))
             elif method == 'DELETE':
-                url = path if raw_path else '%s/%s?%s' % (FACEBOOK_API, path, urllib.urlencode(args))
+                url = path if raw_path else '%s/%s?%s' % (self.api_root, path, urllib.urlencode(args))
                 req = urllib2.Request(url)
                 req.get_method = lambda: 'DELETE'
                 f = urllib2.urlopen(req)
@@ -142,10 +153,13 @@ class AdsAPI(object):
                 raise
             return json.load(f)
         except urllib2.HTTPError as e:
-            print '%s' % e
-            raise AdsAPIError(e)
+            err = AdsAPIError(e)
+            # Info, not warning or error, because these often happen as an expected result because of user input
+            # and well formed requests that facebook rejects.
+            logger.info(u'API Error: {}'.format(err.message))
+            raise err
         except urllib2.URLError as e:
-            print 'URLError: %s' % e.reason
+            logger.warn(u'URLError: %s' % e.reason)
             raise
 
     def make_batch_request(self, batch):
@@ -157,7 +171,7 @@ class AdsAPI(object):
                 for k, v in args.items()}
         logger.info('Making a batched request with %s' % args)
         try:
-            f = urllib2.urlopen(FACEBOOK_API, urllib.urlencode(args))
+            f = urllib2.urlopen(self.api_root, urllib.urlencode(args))
             data = json.load(f)
             # For debugging
             self.data = data
@@ -165,12 +179,11 @@ class AdsAPI(object):
                 data[idx] = json.loads(val['body'])
             return data
         except urllib2.HTTPError as e:
-            print '%s' % e
+            logger.info('%s' % e)
             return json.load(e)
         except urllib2.URLError as e:
-            print 'URLError: %s' % e.reason
+            logger.warn('URLError: %s' % e.reason)
 
-    # New API
     def make_labeled_batch_request(self, batch):
         """Makes a batched request with label against the Facebook Ads API."""
         try:
@@ -206,31 +219,41 @@ class AdsAPI(object):
         args = {'fields': fields} if fields else {}
         return self.make_request(path, 'GET', args, batch=batch)
 
-    def get_adaccounts(self, user_id, fields, batch=False):
+    def get_adaccounts(self, user_id, fields,
+                       paging_cursors={}, batch=False):
         """Returns the list of Facebook ad accounts."""
         path = '%s/adaccounts' % user_id
         args = {'fields': fields}
+        if paging_cursors:
+            args.update(paging_cursors)
         return self.make_request(path, 'GET', args, batch=batch)
 
-    # New API
+    def get_adaccount_transactions(
+            self, account_id, paging_cursors={}, batch=False):
+        """Returns the transactions of the given ad account."""
+        path = 'act_%s/transactions' % account_id
+        args = {}
+        if paging_cursors:
+            args.update(paging_cursors)
+        return self.make_request(path, 'GET', args, batch=batch)
+
+
     def get_adcampaign_group(self, campaign_group_id, fields, batch=False):
         """Return the fields for the given ad campaign group."""
         path = '%s' % campaign_group_id
         args = {'fields': fields}
         return self.make_request(path, 'GET', args, batch=batch)
 
-    # New API
     def get_adcampaign_groups(self, account_id, fields, batch=False):
         """Returns the fields of all ad campaign groups
            from the given ad account."""
-        path = 'act_%s/adcampaign_groups' % account_id
+        path = 'act_%s/campaigns' % account_id
         args = {
             'fields': fields,
             'limit': self.DATA_LIMIT
         }
         return self.make_request(path, 'GET', args, batch=batch)
 
-    # New API
     def delete_adcampaign_group(self, campaign_group_id, batch=False):
         """Delete specific campaign group."""
         path = '%s' % campaign_group_id
@@ -242,19 +265,18 @@ class AdsAPI(object):
         args = {'fields': fields}
         return self.make_request(path, 'GET', args, batch=batch)
 
-    # New API
+    # Deprecated
     def get_adcampaigns_of_campaign_group(self, campaign_group_id, fields,
                                           batch=False):
         """Return the fields of all adcampaigns
            from the given adcampaign group."""
-        path = '%s/adcampaigns' % campaign_group_id
+        path = '%s/adsets' % campaign_group_id
         args = {'fields': fields}
         return self.make_request(path, 'GET', args, batch=batch)
 
-    # New API
     def get_adcampaigns_of_account(self, account_id, fields, batch=False):
         """Returns the fields of all ad sets from the given ad account."""
-        path = 'act_%s/adcampaigns' % account_id
+        path = 'act_%s/adsets' % account_id
         args = {
             'fields': fields,
             'limit': self.DATA_LIMIT
@@ -315,7 +337,6 @@ class AdsAPI(object):
             args = {'hashes': hashes}
         return self.make_request(path, 'GET', args, batch=batch)
 
-    # New API
     def get_stats_by_adcampaign_group(
             self, campaign_group_id, fields=None, filters=None, batch=False,
             start_time=None, end_time=None):
@@ -341,17 +362,19 @@ class AdsAPI(object):
             path = path + '/{0}'.format(self.__parse_time(end_time))
         return iterate_by_page(self.make_request(path, 'GET', args, batch=batch))
 
-    def get_stats_by_adcampaign(self, account_id, campaign_ids=None,
+    def get_stats_by_adcampaign(self, account_id, campaign_ids=None, fields=None,
                                 batch=False, start_time=None, end_time=None):
         """Returns the stats for a Facebook campaign by adcampaign."""
         args = {}
+        if fields:
+            args['fields'] = json.dumps(fields)
         if campaign_ids is not None:
             args['campaign_ids'] = json.dumps(campaign_ids)
         if start_time:
             args['start_time'] = self.__parse_time(start_time)
         if end_time:
             args['end_time'] = self.__parse_time(end_time)
-        path = 'act_%s/adcampaignstats' % account_id
+        path = 'act_%s/adsets' % account_id
         return self.make_request(path, 'GET', args, batch=batch)
 
     def get_stats_by_adgroup(
@@ -368,7 +391,6 @@ class AdsAPI(object):
         path = 'act_%s/adgroupstats' % account_id
         return self.make_request(path, 'GET', args, batch=batch)
 
-    # New API
     def get_time_interval(self, start, end):
         """Returns formatted time interval."""
         if not start or not end:
@@ -384,6 +406,7 @@ class AdsAPI(object):
         )
         return json.dumps(time_interval)
 
+    # Deprecated
     def get_adreport_stats(self, account_id, date_preset, time_increment,
                            data_columns, filters=None, actions_group_by=None,
                            batch=False):
@@ -400,7 +423,7 @@ class AdsAPI(object):
             args['actions_group_by'] = actions_group_by
         return self.make_request(path, 'GET', args, batch=batch)
 
-    # New API
+    # Deprecated
     def get_adreport_stats2(self, account_id, data_columns, date_preset=None,
                             date_start=None, date_end=None,
                             time_increment=None, actions_group_by=None,
@@ -409,7 +432,7 @@ class AdsAPI(object):
                             limit=None):
         """Returns the ad report stats for the given account."""
         if date_preset is None and date_start is None and date_end is None:
-            raise BaseException("Either a date_preset or a date_start/end \
+            raise AdsAPIError("Either a date_preset or a date_start/end \
                                 must be set when requesting a stats info.")
         path = 'act_%s/reportstats' % account_id
         args = {
@@ -441,13 +464,61 @@ class AdsAPI(object):
             return self.make_request(path, 'POST', args=args, batch=batch)
         return self.make_request(path, 'GET', args=args, batch=batch)
 
-    # New API
+    def get_insights(self, object_id, fields=None, action_breakdowns=None,
+                     level=None, date_preset=None, date_start=None,
+                     date_end=None, time_increment=None, breakdowns=None,
+                     filtering=None, sort=None, limit=None,
+                     action_attribution_windows=None,
+                     async=False, batch=False):
+        """Returns the insights information for the given object."""
+        if date_preset is None and date_start is None and date_end is None:
+            raise AdsAPIError("Either a date_preset or a date_start/end \
+                                must be set when requesting a stats info.")
+        path = '%s/insights' % object_id
+        args = {}
+        if fields:
+            args['fields'] = fields
+        if action_breakdowns:
+            args['action_breakdowns'] = json.dumps(action_breakdowns)
+        if action_attribution_windows:
+            args['action_attribution_windows'] = json.dumps(action_attribution_windows)
+        if level:
+            args['level'] = level
+        if date_preset:
+            args['date_preset'] = date_preset
+        if date_start and date_end:
+            args['time_interval'] = \
+                self.get_time_interval(date_start, date_end)
+        if time_increment:
+            args['time_increment'] = time_increment
+        if breakdowns:
+            args['breakdowns'] = json.dumps(breakdowns)
+        if filtering:
+            args['filtering'] = json.dumps(filtering)
+        if sort:
+            args['sort'] = sort
+        if limit:
+            args['limit'] = limit
+        if async:
+            args['async'] = 'true'
+            return self.make_request(path, 'POST', args=args, batch=batch)
+        return self.make_request(path, 'GET', args=args, batch=batch)
+
+    def get_targetingsentencelines(self, object_id, batch=False):
+        """Returns targeting information for the given object."""
+        path = '%s/targetingsentencelines' % object_id
+        return self.make_request(path, 'GET', batch=batch)
+
+    def get_reachestimate(self, object_id, batch=False):
+        """Returns reach estimation for the given object."""
+        path = '%s/reachestimate' % object_id
+        return self.make_request(path, 'GET', batch=batch)
+
     def get_async_job_status(self, job_id, batch=False):
         """Returns the asynchronously requested job status"""
         path = '%s' % job_id
         return self.make_request(path, 'GET', batch=batch)
 
-    # New API
     def get_async_job_result(self, account_id, job_id, batch=False):
         """Returns completed result of the given async job"""
         path = 'act_%s/reportstats' % account_id
@@ -456,6 +527,11 @@ class AdsAPI(object):
         }
         return self.make_request(path, 'GET', args=args, batch=batch)
 
+    def get_async_job_result2(self, job_id, batch=False):
+        """Returns completed result of the given async job"""
+        path = '%s/insights' % job_id
+        return self.make_request(path, 'GET', batch=batch)
+
     def get_conversion_stats_by_adaccount(self, account_id, batch=False):
         """Returns the aggregated conversion stats for the given ad account."""
         path = 'act_%s/conversions' % account_id
@@ -498,10 +574,12 @@ class AdsAPI(object):
         path = '%s/conversions' % adgroup_id
         return self.make_request(path, 'GET', batch=batch)
 
-    def get_custom_audiences(self, account_id, batch=False):
+    def get_custom_audiences(self, account_id, fields=None, batch=False):
         """Returns the information for a given audience."""
         path = 'act_%s/customaudiences' % account_id
-        return self.make_request(path, 'GET', batch=batch)
+        args = { 'limit': self.DATA_LIMIT }
+        if fields: args['fields'] = fields
+        return self.make_request(path, 'GET', args, batch=batch)
 
     def get_ads_pixels(self, account_id, fields=None, batch=False):
         """Returns the remarketing pixel."""
@@ -536,17 +614,18 @@ class AdsAPI(object):
         path = 'act_%s/ratecard' % account_id
         return self.make_request(path, 'GET', batch=batch)
 
-    def get_reach_estimate(self, account_id, currency, targeting_spec,
+    def get_reach_estimate(self, account_id, targeting_spec, currency=None,
                            creative_action_spec=None,
                            bid_for=None, batch=False):
         """Returns the reach estimate for the given currency and targeting."""
         path = 'act_%s/reachestimate' % account_id
         args = {
-            'currency': currency,
-            'targeting_spec': targeting_spec,
+            'targeting_spec': json.dumps(targeting_spec),
         }
+        if currency is  not None:
+            args['currency'] = json.dumps(currency)
         if creative_action_spec is not None:
-            args['creative_action_spec'] = creative_action_spec
+            args['creative_action_spec'] = json.dumps(creative_action_spec)
         if bid_for is not None:
             args['bid_for'] = bid_for
         return self.make_request(path, 'GET', args, batch=batch)
@@ -714,7 +793,6 @@ class AdsAPI(object):
             args['scheduled_publish_time'] = scheduled_publish_time
         return self.make_request(path, 'POST', args, files, batch=batch)
 
-    # New API
     def create_adcampaign_group(self, account_id, name, campaign_group_status,
                                 objective=None, batch=False):
         """Creates an ad campaign group for the given account."""
@@ -727,7 +805,6 @@ class AdsAPI(object):
             args['objective'] = objective
         return self.make_request(path, 'POST', args, batch=batch)
 
-    # New API
     def update_adcampaign_group(self, campaign_group_id, name=None,
                                 campaign_group_status=None, objective=None,
                                 batch=False):
@@ -742,19 +819,20 @@ class AdsAPI(object):
             args['objective'] = objective
         return self.make_request(path, 'POST', args, batch=batch)
 
-    # New API: Need to change 'create_adcampaign' when facebook api is set new api.
-    def _create_adcampaign(self, account_id, campaign_group_id, name,
-                           campaign_status,
-                           daily_budget=None, lifetime_budget=None,
-                           start_time=None, end_time=None, batch=False):
-        """Creates an ad campaign for the given account and
-           the given campaign group."""
+    def create_adset(self, account_id, campaign_group_id, name,
+                     campaign_status, daily_budget=None, lifetime_budget=None,
+                     start_time=None, end_time=None,
+                     bid_type=None, bid_info=None, promoted_object=None, targeting=None, batch=False):
+        """
+        Creates an adset (formerly called ad campaign) for the given account and the campaign (formerly called "campaign group").
+        """
         if daily_budget is None and lifetime_budget is None:
-            raise BaseException("Either a lifetime_budget or a daily_budget \
+            raise AdsAPIError("Either a lifetime_budget or a daily_budget \
                                 must be set when creating a campaign")
         if lifetime_budget is not None and end_time is None:
-            raise BaseException("end_time is required when lifetime_budget \
+            raise AdsAPIError("end_time is required when lifetime_budget \
                                 is specified")
+
         path = 'act_%s/adcampaigns' % account_id
         args = {
             'campaign_group_id': campaign_group_id,
@@ -769,54 +847,22 @@ class AdsAPI(object):
             args['start_time'] = start_time
         if end_time:
             args['end_time'] = end_time
+        if bid_type:
+            args['bid_type'] = bid_type
+        if bid_info:
+            args['bid_info'] = bid_info
+        if promoted_object:
+            args['promoted_object'] = json.dumps(promoted_object)
+        if targeting:
+            args['targeting'] = json.dumps(targeting)
+
         return self.make_request(path, 'POST', args, batch=batch)
 
-    def create_adset(self, account_id, campaign_group_id, name,
-                     campaign_status, daily_budget=None, lifetime_budget=None,
-                     start_time=None, end_time=None, batch=False):
-        """
-        Creates an ad campaign for the given account and the campaign group.
-        Functionality of this method is same as _create_adcampaign method.
-        """
-        return self._create_adcampaign(
-            account_id, campaign_group_id, name, campaign_status,
-            daily_budget, lifetime_budget, start_time, end_time, batch)
-
-    # Deprecated: this method will be update.
-    def create_adcampaign(self, account_id, name, campaign_status,
+    def update_adset(self, campaign_id, name=None, campaign_status=None,
                           daily_budget=None, lifetime_budget=None,
-                          start_time=None, end_time=None, batch=False):
-        """
-        Creates an ad campaign for the given account.
-        Deprecated: This method cannot work on new campaign structure.
-        """
-        logger.warn("This method is deprecated.")
-        if daily_budget is None and lifetime_budget is None:
-            raise BaseException("Either a lifetime_budget or a daily_budget \
-                                 must be set when creating a campaign")
-        if lifetime_budget is not None and end_time is None:
-            raise BaseException("end_time is required when lifetime_budget \
-                                is specified")
-        path = 'act_%s/adcampaigns' % account_id
-        args = {
-            'name': name,
-            'campaign_status': campaign_status,
-        }
-        if daily_budget:
-            args['daily_budget'] = daily_budget
-        if lifetime_budget:
-            args['lifetime_budget'] = lifetime_budget
-        if start_time:
-            args['start_time'] = start_time
-        if end_time:
-            args['end_time'] = end_time
-        return self.make_request(path, 'POST', args, batch=batch)
-
-    # New API
-    def update_adcampaign(self, campaign_id, name=None, campaign_status=None,
-                          daily_budget=None, lifetime_budget=None,
-                          start_time=None, end_time=None, batch=False):
-        """Updates condition of the given ad campaign."""
+                          start_time=None, end_time=None,
+                          bid_type=None, bid_info=None, promoted_object=None, targeting=None, batch=False):
+        """Updates the given adset."""
         path = '%s' % campaign_id
         args = {}
         if name:
@@ -829,47 +875,56 @@ class AdsAPI(object):
             args['lifetime_budget'] = lifetime_budget
         if start_time:
             args['start_time'] = start_time
-        if end_time:
+        if end_time is not None:
             args['end_time'] = end_time
+        if bid_type:
+            args['bid_type'] = bid_type
+        if bid_info:
+            args['bid_info'] = bid_info
+        if promoted_object:
+            args['promoted_object'] = json.dumps(promoted_object)
+        if targeting:
+            args['targeting'] = json.dumps(targeting)
+
         return self.make_request(path, 'POST', args, batch=batch)
 
-    # New API
     def delete_adcampaign(self, campaign_id, batch=False):
         """Delete the given ad campaign."""
         path = '%s' % campaign_id
         return self.make_request(path, 'DELETE', batch=batch)
 
-        logger.warn("This method is deprecated and is replaced with get_ads_pixels.")
-
-    def create_adcreative(self, account_id, object_story_id=None, batch=False):
+    def create_adcreative(self, account_id, name=None, object_story_id=None, object_story_spec=None, batch=False):
         """Creates an ad creative in the given ad account."""
         path = 'act_%s/adcreatives' % account_id
-        args = {
-            'object_story_id': object_story_id,
-        }
-        args = {k: v for k, v in args.iteritems() if v is not None}
+        args = {}
+        if name:
+            args['name'] = name
+        if object_story_id:
+            args['object_story_id'] = object_story_id
+        if object_story_spec:
+            args['object_story_spec'] = json.dumps(object_story_spec)
+
         return self.make_request(path, 'POST', args, batch=batch)
 
-    def create_adgroup(self, account_id, name, bid_type, bid_info, campaign_id,
-                       creative_id, targeting, max_bid=None, conversion_specs=None,
+    def create_adgroup(self, account_id, name, campaign_id,
+                       creative_id, bid_type=None, bid_info=None, max_bid=None,
                        tracking_specs=None, view_tags=None, objective=None,
-                       adgroup_status=None, batch=False):
+                       adgroup_status=None, targeting=None, conversion_specs=None, batch=False):
         """Creates an adgroup in the given ad camapaign with the given spec."""
         path = 'act_%s/adgroups' % account_id
         args = {
             'name': name,
-            'bid_type': bid_type,
-            'bid_info': json.dumps(bid_info),
             'campaign_id': campaign_id,
             'creative': json.dumps({'creative_id': creative_id}),
-            'targeting': json.dumps(targeting),
         }
+        if bid_type:
+            args['bid_type'] = bid_type
         if max_bid:
-            assert bid_type == 'CPM', 'can only use max_bid with CPM bidding'
+            # can only use max_bid with CPM bidding
             args['max_bid'] = max_bid
-            del args['bid_info']  # get rid of bid_info
-        if conversion_specs:
-            args['conversion_specs'] = json.dumps(conversion_specs)
+        elif bid_info:
+            args['bid_info'] = json.dumps(bid_info)
+
         if tracking_specs:
             args['tracking_specs'] = json.dumps(tracking_specs)
         if view_tags:
@@ -878,12 +933,16 @@ class AdsAPI(object):
             args['objective'] = objective
         if adgroup_status:
             args['adgroup_status'] = adgroup_status
+        if targeting:
+            args['targeting'] = json.dumps(targeting)
+        if conversion_specs:
+            args['conversion_specs'] = json.dumps(conversion_specs)
         return self.make_request(path, 'POST', args, batch=batch)
 
     def update_adgroup(self, adgroup_id, name=None, adgroup_status=None,
                        bid_type=None, bid_info=None, creative_id=None,
-                       targeting=None, conversion_specs=None,
                        tracking_specs=None, view_tags=None, objective=None,
+                       targeting=None, conversion_specs=None,
                        batch=False):
         """Updates condition of the given ad group."""
         path = "%s" % adgroup_id
@@ -894,12 +953,9 @@ class AdsAPI(object):
             args['bid_type'] = bid_type
         if bid_info:
             args['bid_info'] = json.dumps(bid_info)
+
         if creative_id:
             args['creative'] = json.dumps({'creative_id': creative_id})
-        if targeting:
-            args['targeting'] = json.dumps(targeting)
-        if conversion_specs:
-            args['conversion_specs'] = json.dumps(conversion_specs)
         if tracking_specs:
             args['tracking_specs'] = json.dumps(tracking_specs)
         if view_tags:
@@ -908,14 +964,16 @@ class AdsAPI(object):
             args['objective'] = objective
         if adgroup_status:
             args['adgroup_status'] = adgroup_status
+        if targeting:
+            args['targeting'] = json.dumps(targeting)
+        if conversion_specs:
+            args['conversion_specs'] = json.dumps(conversion_specs)
         return self.make_request(path, 'POST', args, batch=batch)
 
-    # Deprecated: this method will be update at Oct 1st 2014, breaking change on Facebook..
     def create_custom_audience(self, account_id, name, subtype=None,
                                description=None, rule=None, opt_out_link=None,
                                retention_days=30, batch=False):
         """Create a custom audience for the given account."""
-        logger.warn("This method is deprecated. This method will be changed in order to support new rule of facebook ads api.")
         path = "act_%s/customaudiences" % account_id
         args = {
             'name': name,
@@ -933,17 +991,21 @@ class AdsAPI(object):
         return self.make_request(path, 'POST', args, batch=batch)
 
     def add_users_to_custom_audience(self, custom_audience_id, tracking_ids,
-                                     schema='MOBILE_ADVERTISER_ID', batch=False):
+                                     schema='MOBILE_ADVERTISER_ID', app_ids=None, batch=False):
         """
         Adds users to a Custom Audience, based on a list of unique user
         tracking ids. There is a limit imposed by Facebook that only 10000
         users may be uploaded at a time.
         @param schema Allowed values are "UID", "EMAIL_SHA256", "PHONE_SHA256",
             "MOBILE_ADVERTISER_ID"
+        @param app_ids List of app ids. This is required for schema type UID, as of API v2.2
         """
         path = "%s/users" % custom_audience_id
+        payload = {'schema': schema, 'data': tracking_ids}
+        if app_ids:
+            payload['app_ids'] = app_ids
         args = {
-            'payload': {'schema': schema, 'data': json.dumps(tracking_ids)}
+            'payload': json.dumps(payload)
         }
         return self.make_request(path, 'POST', args, batch)
 
